; Programmer: Johnny Pan
; Date: 12/04/98 - 08/13/99
; Filename: DRAM.ASM
;
;---------------------------
;
;       GetDRAMSize()
;
;       Output:
;         bx -> Exact Display Memory Size
;
;---------------------------
GetDRAMSize     proc    near

                push    dx
                push    ax

                mov     al,014h
                call    GetSeq
                and     ah,03Fh
                movzx   bx,ah
                inc     bx

                pop     ax
                pop     dx
                ret

GetDRAMSize     endp



;--------------------
;
;       GetMEMSize()
;
;       Output:
;         cx -> Memory Size
;
;--------------------
GetMEMSize      proc    near

                call    GetDRAMSize
                xor     dx,dx
                shl     bx,cl
                mov     cx,bx

                movzx   ax,byte ptr cs:[si+Ext_VESAMEMSize]
                shl     ax,4
                cmp     cx,ax
                jbe     short @F
                mov     cx,ax
@@:             ret

GetMEMSize      endp



;------------------------
;
;       GetVESAMEMSize()
;
;       Output:
;         ebx -> VESA Memory Size
;
;------------------------
GetVESAMEMSize  proc    near

                push    cx
                push    ax

                mov     cl,4
                call    GetMEMSize
                movzx   ebx,cx

                pop     ax
                pop     cx
                ret

GetVESAMEMSize  endp



;------------------------
;
;       SetMemoryClock()
;
;       Description : Set MCLK and ECLK according the DRAM type
;
;------------------------
SetMemoryClock  proc    near

                mov     dx,SEQ_INDEX
                call    GetMCLKPtr
                mov     bx,02A28h
                call    SetBlock

ifdef           SIS301
                call    GetECLKPtr
                mov     bx,0302Eh
                call    SetBlock
endif
                ret

SetMemoryClock  endp



;--------------------
;
;       GetMCLKPtr()
;
;       Description : Get MCLK pointer according to DRAM type
;
;--------------------
GetMCLKPtr      proc    near

                mov     di,offset cs:MCLKData
                call    GetDRAMType
                mov     al,5
                mul     bl
                add     di,ax
                ret

GetMCLKPtr      endp



ifdef           SIS301
;--------------------
;
;       GetECLKPtr()
;
;       Description : Get ECLK pointer according to DRAM type
;
;--------------------
GetECLKPtr      proc    near

                mov     di,offset cs:ECLKData
                call    GetDRAMType
                mov     al,5
                mul     bl
                add     di,ax
                ret

GetECLKPtr      endp
endif



;--------------------
;
;       GetMCLK()
;
;       Output:
;         bx -> Value of MCLK in unit of MHz
;
;--------------------
GetMCLK         proc    near

                push    di
                push    ax

                call    GetMCLKPtr
                mov     bx,word ptr cs:[di+3]

                pop     ax
                pop     di
                ret

GetMCLK         endp



;-------------------------------------------------------------------
;
;       Module Name: Set_Shared_Mem()
;       Description: Set DRAM Timing, Size under UMA
;       Note       : DRAM Type : SDRAM
;                  : DRAM Size : 2, 4, 8 MB
;
;-------------------------------------------------------------------
SetDRAMConfig    proc    near

                 mov     ax,0B102h       ; Find PCI Device
                 mov     cx,DeviceID
                 mov     dx,VendorID
                 xor     si,si
                 int     1Ah

                 mov     cx,bx          ; BH-Bus Number (0..255)
                                        ; BL-Device Number in upper 5bits,
                                        ; Function Number in bottom 3 bits

                 mov     ax,0F401h      ; Call VESA VUMA function call 05h
                 mov     bl,005h        ; Get Current Memory Information
                 int     15h            ; Return :
                                        ;       AX : Status
                                        ;       EBX[3:0] : M/B frequency 0000 : 66MHz
                                        ;                                0001 : 75MHz
                                        ;                                0010 : 83MHz
                                        ;                                0011 :100MHz
                                        ;
                                        ;       EBX[8] : Bus width       0 : 64 bits
                                        ;                                1 : 128 bits
                                        ;
                                        ;       DX[31:0]  : Memory size in 64K-bytes


                 shr      dx,4          ; memory size
                 dec      dx
                 and      dl,003Fh
                 mov      ah,dl

                 shl      bh,007        ; bus width
                 or       ah,bh
                 shr      bh,1
                 xor      bh,040h
                 or       ah,bh
                 mov      al,014h
                 call     SetSeq

                 mov      ah,bl         ; M/B frequency
                 and      ah,00Fh
                 mov      al,01Ah
                 mov      bl,0F0h
                 call     SetSeqANDOR

                 mov      ah,cl         ; Display I
                 mov      al,017h
                 call     SetSeq

                 mov      ah,ch         ; PanelID, DVI
                 and      ah,03Fh
                 mov      al,018h
                 call     SetSeq


                 shr      ecx,1         ; Display II
                 mov      ah,ch
                 and      ah,0E0h
                 mov      al,16h
                 call     SetSeq

                 ret

SetDRAMConfig    endp
