; Programmer: Johnny Pan
; Date: 11/17/98 - 08/13/99
; Filename: UTIL.ASM
;
;---------------------------------------
;
;       ModeType()
;
;       Output:
;         Zero Flag -> 0 not color mode
;                      1 mono mode
;
;---------------------------------------
ModeType        proc    near

                cmp     al,07h
                je      @F
                cmp     al,0Fh
@@:             ret

ModeType        endp



;-------------------------
;
;       SetSeq()
;       SetCR()
;       SetReg()
;
;       Input:
;          dx -> I/O port
;          al -> index
;          ah -> data
;
;-------------------------
SetSeq          proc    near

                mov     dx,SEQ_INDEX
                jmp     short SetReg
SetCR:          mov     dx,word ptr ADDR_6845
SetReg:         cmp     dx,COL_INDEX
                ja      short @F
                out     dx,ax
                ret

@@:             out     dx,al
                jmp     $+2
                inc     dx
                xchg    al,ah
                out     dx,al
                jmp     $+2
                xchg    al,ah
                dec     dx
                ret

SetSeq          endp



;-------------------------
;
;       GetSeq()
;       GetCR()
;       GetReg()
;
;       Input:
;          dx -> I/O port
;          al -> index
;       Output:
;          ah -> data
;
;-------------------------
GetSeq          proc    near

                mov     dx,SEQ_INDEX
                jmp     short GetReg
GetCR:          mov     dx,word ptr ADDR_6845
GetReg:         cmp     dx,COL_INDEX
                ja      short @F
                out     dx,al
                in      ax,dx
                ret

@@:             out     dx,al
                jmp     $+2
                inc     dx
                mov     ah,al
                in      al,dx
                jmp     $+2
                dec     dx
                xchg    al,ah
                ret

GetSeq          endp



;-------------------------
;
;       SetSeqAND()
;       SetCRAND()
;       SetRegAND()
;
;       Input:
;          dx -> I/O port
;          al -> index
;          ah -> AND data
;
;-------------------------
SetSeqAND       proc    near

                mov     dx,SEQ_INDEX
                jmp     short SetRegAND
SetCRAND:       mov     dx,word ptr ADDR_6845
SetRegAND:      push    bx
                push    ax
                mov     bh,ah
                call    GetReg
                and     ah,bh
                call    SetReg
                pop     ax
                pop     bx
                ret

SetSeqAND       endp



;-------------------------
;
;       SetSeqOR()
;       SetCROR()
;       SetRegOR()
;
;       Input:
;          dx -> I/O port
;          al -> index
;          ah -> OR data
;
;-------------------------
SetSeqOR        proc    near

                mov     dx,SEQ_INDEX
                jmp     short SetRegOR
SetCROR:        mov     dx,word ptr ADDR_6845
SetRegOR:       push    bx
                push    ax
                mov     bh,ah
                call    GetReg
                or      ah,bh
                call    SetReg
                pop     ax
                pop     bx
                ret

SetSeqOR        endp



;-------------------------
;
;       SetSeqANDOR()
;       SetCRANDOR()
;       SetRegANDOR()
;
;       Input:
;          dx -> I/O port
;          al -> index
;          ah -> OR data
;          bl -> and data
;
;-------------------------
SetSeqANDOR     proc    near

                mov     dx,SEQ_INDEX
                jmp     short SetRegANDOR
SetCRANDOR:     mov     dx,word ptr ADDR_6845
SetRegANDOR:    push    bx
                push    ax
                mov     bh,ah
                call    GetReg
                and     ah,bl
                or      ah,bh
                call    SetReg
                pop     ax
                pop     bx
                ret

SetSeqANDOR     endp



;---------------------
;
;       SetWordRegs()
;
;---------------------
SetWordRegs     proc    near

                mov     ah,bh
                call    SetReg
                inc     al
                mov     ah,bl
                call    SetReg
                ret

SetWordRegs     endp



;---------------------------------------------
;
;       SetBlock()
;
;       Input:
;          bl -> setting start register index
;          bh -> setting end register index
;          es:di -> setting parameter
;
;---------------------------------------------
SetBlock        proc    near

@@:             mov     al,bl
                mov     ah,byte ptr es:[di]
                inc     di
                call    SetReg
                inc     bl
                cmp     bl,bh
                jbe     short @B
                ret

SetBlock        endp



;---------------------------------------------
;
;       ReadBlock()
;
;       Input:
;          bl -> setting start register index
;          es:di -> reading parameter
;
;---------------------------------------------
ReadBlock       proc    near

ReadBlockLp:    mov     al,bl
                call    GetReg
                xchg    ah,al
                stosb
                inc     bl
                loop    ReadBlockLp
                ret

ReadBlock       endp



;------------------------
;
;       UnLockCR0ToCR7()
;
;------------------------
UnLockCR0ToCR7  proc

                mov     ax,07F11h
                call    SetCRAND
                ret

UnLockCR0ToCR7  endp



;----------------------
;
;       LockCR0ToCR7()
;
;----------------------
LockCR0ToCR7    proc

                mov     ax,08011h
                call    SetCROR
                ret

LockCR0ToCR7    endp



DisplayOn:      xor     ah,ah
                jmp     short DisplayOnOff
DisplayOff:     mov     ah,020h
;----------------------
;
;       DisplayOnOff()
;
;----------------------
DisplayOnOff    proc    near

                mov     al,001h
                mov     bl,not 020h
                call    SetSeqANDOR
                ret

DisplayOnOff    endp



;-------------------
;
;       ResetATTR()
;
;-------------------
ResetATTR       proc    near

                mov     dx,COL_INDEX
                add     dx,06h
                in      al,dx
                xor     dl,060h
                in      al,dx
                ret

ResetATTR       endp



;--------------------
;
;       EnableATTR()
;
;--------------------
EnableATTR      proc    near

                call    ResetATTR

                mov     dx,ATTREG
                mov     al,20h
                out     dx,al

                call    ResetATTR
                ret

EnableATTR      endp



;-------------------------
;
;       WaitSavePalette()
;
;-------------------------
WaitSavePalette proc    near

                call    LongWait
                call    SetPalette
ifdef           AUX
                call    SaveAuxPalette
endif
                call    EnableATTR
                ret

WaitSavePalette endp



;-------------------------
;
;       WaitReadPalette()
;
;-------------------------
WaitReadPalette proc    near

                call    LongWait
                call    GetPalette
                call    EnableATTR
                ret

WaitReadPalette endp



;---------------------
;
;       SetPalette()
;
;       Input:
;         bl -> index
;         bh -> data
;
;---------------------
SetPalette      proc    near

                pushf

                cli
                call    ReSetATTR
                mov     dx,ATTREG
                mov     al,bl
                out     dx,al
                mov     al,bh
                out     dx,al

                popf
                ret

SetPalette      endp



;---------------------
;
;       GetPalette()
;
;       Input:
;         bl -> index
;       Outputs:
;         ah -> data
;
;---------------------
GetPalette      proc    near

                pushf

                cli
                call    ReSetATTR
                mov     dx,ATTREG
                mov     al,bl
                out     dx,al
                inc     dl
                in      al,dx
                mov     ah,al

                popf
                ret

GetPalette      endp



ifdef           AUX
;------------------------
;
;       SaveAuxPalette()
;
;       Input:
;         bl -> index
;         bh -> data
;
;------------------------
SaveAuxPalette  proc    near

                push    ds
                push    bx

                cmp     bl,11h
                jne     short @F
                mov     bl,010h

@@:             lds     si,dword ptr SAVE_PTR
                lds     si,dword ptr ds:[si+04h]
                mov     ax,ds
                or      ax,si
                jz      short @F
                movzx   ax,bl
                add     si,ax
                mov     byte ptr ds:[si],bh

@@:             pop     bx
                pop     ds
                ret

SaveAuxPalette  endp
endif



;------------------
;
;       LongWait()
;
;------------------
LongWait        proc    near

                push    dx
                push    cx
                push    ax

                mov     al,1Fh                  ;;[ycchen] 02/11/00
                call    GetSeq                  ; work-a-round (Ref. Item 22)
                test    ah,0C0h
                jnz     short LongWaitRet

                mov     dx,word ptr ADDR_6845
                add     dx,06h
                xor     cx,cx
@@:             in      al,dx
                test    al,08h
                loopnz  short @B

                xor     cx,cx
@@:             in      al,dx
                test    al,08h
                loopz   short @B

LongWaitRet:    pop     ax
                pop     cx
                pop     dx
                ret

LongWait        endp

;--------------------
;
;       SetEDIBank()
;
;--------------------
SetEDIBank      proc    near

                push    edi
                push    ax

                shr     edi,16
                mov     dx,di
                call    SetBank

                pop     ax
                pop     edi
                ret

SetEDIBank      endp



;--------------------
;
;       SetESIBank()
;
;--------------------
SetESIBank      proc    near

                push    esi
                push    ax

                shr     esi,16
                mov     dx,si
                call    SetReadBank

                pop     ax
                pop     esi
                ret

SetESIBank      endp



;---------------------
;
;       Delay()
;
;       Input:
;         cx -> delay
;
;---------------------
Delay           proc    near

                push    dx
                push    cx
                push    ax

DelayLp:        mov     dx,SEQ_INDEX
                mov     al,005h
                call    GetReg
                loop    DelayLp

                pop     ax
                pop     cx
                pop     dx
                ret

Delay           endp



